/**
 * Interface for server side communication
 **/
jsp.api = (function(){

    var pluginName = location.pathname.split("/")[2],
        dunePhpUrl = '/cgi-bin/plugins/' + pluginName + '/do_php/',
        proxyUrl = {
            json: dunePhpUrl + 'curl-api-proxy.php?path=',
            xml: dunePhpUrl + 'xml_parser.php?path='
        };

    var CONFIG_URL = 'http://127.0.0.1/plugins/' + pluginName + '/config.xml',
        CONFIG_TYPE = 'xml';

    var channels = {},
        channelsArr = [],
        options;

    function loadData(params){
        loadConfig({callbackSuccess: function() {
            loadChannelsInfo({callbackSuccess: loadPrograms.bind(null, params)});
        }});
    }

    function loadPrograms(params){
        if (!isEpgEnable()) {
            params.callbackSuccess();
            return;
        }

        var loadedChannelsCount = 0;

        function loadCallback() {
            loadedChannelsCount++;
            if (loadedChannelsCount === len && params.callbackSuccess) {
                params.callbackSuccess();
            }
        }

        for (var i = 0, len = channelsArr.length, item; i < len; i++) {
            item = channelsArr[i];
            if (item.program_url) {
                loadChannelProgram({channel: channelsArr[i].id, callbackSuccess: loadCallback, callbackError: loadCallback});
            } else {
                loadCallback();
            }
        }
    }

    function savePlayChannel(params) {
        getDuneapi().setUserSetting(pluginName + "playChannel", params.channel);
    }

    function loadConfig(params) {
        var url = proxyUrl[CONFIG_TYPE] + CONFIG_URL;
        jsp.ajax.get(url, function(res) {
            options = res;

            params.callbackSuccess && params.callbackSuccess();
        }, params.callbackError);
    }

    function loadChannelsInfo(params) {
        var url = proxyUrl[options.channel_type] + options.channel_url;
        jsp.ajax.get(url, function(res) {
            channelsArr = res.tv_channels.tv_channel;

            var fw_version = getDuneapi().getFirmwareVersion();
            if (channelsArr.length > 2 && fw_version < "131019_0201") {
                channelsArr.length = channelsArr.length - 2;
            }

            for (var i = 0, len = channelsArr.length, item; i < len; i++) {
                item = channelsArr[i];
                item.number = i;
                channels[item.id] = item;
            }

            params.callbackSuccess && params.callbackSuccess();
        }, params.callbackError);
    }

    function loadChannelProgram(params) {
        var channel = channels[params.channel];

        var url = proxyUrl[channel.program_type] + channel.program_url;
        jsp.ajax.get(url, function(res) {
            updatePrograms(res.programs_item);
            channels[params.channel].programs = res.programs_item;

            params.callbackSuccess && params.callbackSuccess();
        }, params.callbackError);
    }

    function updatePrograms(items) {
        for (var i = 0, len = items.length, item; i < len; i++) {
            item = items[i];
            item.startDate = new Date(item.start_time*1000);
            item.stopDate = new Date(item.stop_time*1000);
            item.date = jsp.dateToString(item.startDate);
            item.time = jsp.timeToString(item.startDate);
        }
    }

    function imagePreload(){
        var img = document.createElement("img"),
            index = 0;

        img.onload = changeSrc;

        function changeSrc(){
            var channel = channelsArr[index];

            if (channel) {
                setTimeout(function() {
                    img.src = channel.icon_url;
                }, 100);
            }
            index++;
        }

        changeSrc();
    }

    function getProgramsByDate(channel, dateTime) {
        var programs = channels[channel].programs || [],
            datePrograms = [],
            date = jsp.dateToString(dateTime),
            item;

        for (var i = 0, len = programs.length; i < len; i++) {
            item = programs[i];
            if (item.date === date) {
                datePrograms.push(item);
            }
        }

        return datePrograms;
    }

    function findProgramByTime(channel, dateTime) {
        var programs = channels[channel].programs || [];

        for (var i = 0, len = programs.length, item; i < len; i++ ) {
            item = programs[i];

            if (item.startDate <= dateTime && dateTime < item.stopDate) {
                return item;
            }
        }

        return null;
    }

    function nextProgram(channel, prevProgram) {
        var programs = channels[channel].programs || [],
            programIndex = programs.indexOf(prevProgram);

        return programs[programIndex + 1] || null;
    }

    function getChannelInfo(channel) {
        return channels[channel] || null;
    }

    function getChannelInfoByNumber(number) {
        return channelsArr[number] || null;
    }

    function getFirstChannel() {
        return channelsArr[0] || null;
    }

    function getLastChannel() {
        return channelsArr[channelsArr.length-1] || null;
    }

    function getChannelsList() {
        return channelsArr;
    }

    function getPlayingChannelId() {
        var id = getDuneapi().getUserSetting(pluginName + "playChannel");
        if (!id) {
            return getFirstChannel().id;
        }
        for (var i = 0, len = channelsArr.length; i < len; i++) {
            if (id == channelsArr[i].id) {
                return id;
            }
        }
        return getFirstChannel().id;
    }

    function getDuneapi() {
        var api = jsp.byId("duneapi");
        if (!api || !api.getUserSetting) {
            api = {getUserSetting: function(){}, setUserSetting: function(){}};
        }
        return api;
    }

    function getExitPluginKey() {
        return options.exit_plugin_key;
    }

    function isEpgEnable() {
        return options.epg_enable === "true";
    }

    function getChannelsCount() {
        return channelsArr.length;
    }

    return {
        getProgramsByDate: getProgramsByDate,
        findProgramByTime: findProgramByTime,
        getChannelInfo: getChannelInfo,
        getChannelInfoByNumber: getChannelInfoByNumber,
        nextProgram: nextProgram,
        loadData: loadData,
        getChannelsList: getChannelsList,
        getFirstChannel: getFirstChannel,
        getLastChannel: getLastChannel,
        getPlayingChannelId: getPlayingChannelId,
        savePlayChannel: savePlayChannel,
        getExitPluginKey: getExitPluginKey,
        isEpgEnable: isEpgEnable,
        imagePreload: imagePreload,
        getChannelsCount: getChannelsCount
    }
})();
