/**
 * Perform an asynchronous HTTP (Ajax) request.
 */
jsp.ajax = (function () {

    var requestTimeout =  5000;

    function _error() {
        jsp.manager.log("IP_TV_SAMPLE: server not response");
    }
    function _success() {}

    function _parseResponse(responseText, successCB, errorCB, type) {
        var response_object = null;
        try {
            if (type === "xml") {
                var parser = new DOMParser();
                response_object = parser.parseFromString(responseText, "text/xml");
            } else {
                response_object = JSON.parse(responseText);
            }
        } catch(e) {
            errorCB("Incorrect response");
        }
        successCB(response_object);
    }

    function _get(url, success, error, type) {
        var errorCB = error || _error,
            successCB = success || _success;

        return _http_request(url, successCB, errorCB, type);
    }


    function _http_request(url, success, error, type) {
        var errorCB = error || _error,
            successCB = success || _success,
            errorTimer,
            xmlhttp;

        errorTimer = setTimeout(function() {
            xmlhttp.abort();
            showError();
        }, requestTimeout);

        xmlhttp = new XMLHttpRequest();
        xmlhttp.open('GET', url);

        xmlhttp.onreadystatechange = function() {
            if (xmlhttp.readyState == 4) {
                clearTimeout(errorTimer);

                if (xmlhttp.status != 200) {
                    xmlhttp.abort();
                    showError();
                } else {
                    _parseResponse(xmlhttp.responseText, successCB, errorCB, type);
                }
            }
        };

        xmlhttp.send();

        function showError() {
            errorCB("Communication error");
        }
    }

    return {
        get: _get
    }
})();