/**
 * Auxiliary functions for working with the DOM, time and xml
 **/

jsp.DEFAULT_KEY_EVENT = "keydown";
jsp.TIMEZONE_OFFSET = 0;
jsp.DAY_NAMES = ['Воскресенье', 'Понедельник', 'Вторник', 'Среда', 'Четверг', 'Пятница', 'Суббота'];
jsp.MILISEC_IN_DAY = 86400000;


jsp.extend = function(Child, Parent) {
    var F = function() { };
    F.prototype = Parent.prototype;
    Child.prototype = new F();
    Child.prototype.constructor = Child;
    Child.superclass = Parent.prototype;
};

jsp.open  = function(viewName, params) {
    var hash = viewName;
    if (params && params.length) {
        hash += "/" + jsp.arrayToURI(params);
    }
    if (location.hash != ("#" + hash) && location.hash != hash) {
        location.hash = hash;
    }
};

jsp.arrayToURI  = function(arr) {
    var result = "",
        separator = "";

    for(var i = 0, len = arr.length; i < len; i++) {
        result += separator + encodeURIComponent(arr[i]);
        separator = "/";
    }

    return result;
};

jsp.uriToArr  = function(hash) {
    var result = hash.split("/");

    for(var i = 0, len = result.length; i < len; i++) {
        result[i] = decodeURIComponent(result[i]);
    }

    return result;
};


/* --------------------------------- DOM ----------------------------- */
/**
 * Find element by id from DOM
 */
jsp.byId = function(id){
    return document.getElementById(id);
};

/**
 * Find element by className from DOM or element
 */
jsp.byClass = function(className, element){
    var obj = element || document;
    return obj.getElementsByClassName(className)[0];
};

jsp.hide = function (element) {
    element.classList.add("hidden");
};

jsp.show = function (element) {
    element.classList.remove("hidden");
};

jsp.changeVisible = function (element, visible) {
   if(visible) {
       jsp.show(element);
   } else {
       jsp.hide(element);
   }
};

jsp.toggle = function (element) {
    if (element.classList.contains("hidden")) {
        jsp.show(element);
    } else {
        jsp.hide(element);
    }
};

jsp.isVisible = function (element) {
    return !element.classList.contains("hidden");
};

jsp.toggleClass = function (element, className) {
    if (element.classList.contains(className)) {
        element.classList.remove(className);
    } else {
        element.classList.add(className);
    }
};

jsp.createNode = function (tag, attr){
    var tagName = tag || "div",
        el = document.createElement(tagName);

    if(attr){
        for(name in attr){
            if(name == 'class' || name == 'className'){
                el.className = attr[name];
            }else if(name == 'html'){
                el.innerHTML = attr[name];
            }else{
                el.setAttribute(name, attr[name]);
            }
        }
    }
    return el;
};

/* --------------------------------- TIME ----------------------------- */

jsp.stringToDate = function(str){
    var date = new Date(str);
    date.setTime(date.getTime() + ( (-date.getTimezoneOffset() - jsp.TIMEZONE_OFFSET) * 60 * 1000));

    return date;
};

jsp.timeToString = function(date){
    var h = date.getHours(),
        m = date.getMinutes();

    return (h < 10 ? "0" + h : h) + ":" + (m < 10 ? "0" + m : m);
};

jsp.timeWithSecToString = function(date){
    var h = date.getHours(),
        m = date.getMinutes(),
        s = date.getSeconds();

    return (h < 10 ? "0" + h : h) + ":" + (m < 10 ? "0" + m : m) + ":" + (s < 10 ? "0" + s : s);
};

jsp.dateToString = function(date){
    var y = date.getFullYear(),
        m = date.getMonth() + 1,
        d = date.getDate();

    return y + "-" + (m < 10 ? "0" + m : m) + "-" + (d < 10 ? "0" + d : d);
};

jsp.dateToDayName = function(date){
    var day = date.getDay();

    return jsp.DAY_NAMES[day];
};

jsp.addDaysToDate = function(date, days){
    return new Date(date.getTime() + days * jsp.MILISEC_IN_DAY);
};

/* --------------------------------- MXL ----------------------------- */

jsp.getXMLNode = function (xmlNode, nodeName){
    var nodes = jsp.getXMLNodes(xmlNode, nodeName);
    return (nodes) ? nodes[0] : {};
};

jsp.getXMLNodes = function (xmlNode, nodeName){
    return xmlNode.getElementsByTagName(nodeName);
};