/**
 * Second page. TV player and programs for a channels.
 */
jsp.views.video = function() {
    jsp.views.video.superclass.constructor.apply(this, arguments);

    this.CLOCK_UPDATE_DURATION = 1000;
    this.HIDE_INFO_DURATION = 5000;
    this.PROGRAM_NOT_FIND = "Список телепередач недоступен для этого канала";

    this.epgIsVisible = jsp.api.isEpgEnable();

    this.viewDiv.innerHTML =
        '<div class="channel-stream"><div class="video" autoplay tabindex="1" data-left="empty" data-right="empty" data-up="changeChannelUp" data-down="changeChannelDown" data-enter="showChannels" data-popupmenu="showChannels"></div></div>' +
        '<div class="info">' +
            '<div class="clock"></div>' +
            '<div class="play-channel"></div>' +
            '<div class="play-programs"></div>' +
        '</div>' +
        '<div class="container channels hidden' + (this.epgIsVisible ? '' : ' big') + '">' +
            '<div class="title">Каналы</div>' +
            '<div class="list">' + generateChannelsHTML() + '</div>' +
            '<div class="scroll-btn"></div>' +
        '</div>' +
        '<div class="container programs hidden">' +
            '<div class="title"></div>' +
            '<div class="prev"></div><div class="next"></div>' +
            '<div class="list"></div>' +
            '<div class="scroll-btn"></div>' +
        '</div>';

    function generateChannelsHTML(){
        var channels = jsp.api.getChannelsList(),
            html = '';

        for (var i = 0, len = channels.length, item; i < len; i++) {
            item = channels[i];
            html += '<div tabindex="1" id="' + item.id + '" tabindex="1" data-left="empty" data-right="empty" data-enter="openChannel" data-up="channelsListUp" data-down="channelsListDown" class="item">' +
                        '<div class="numb">' + (item.number + 1) + '</div>' +
                        (item.icon_url ? '<div class="img" style="background-image: url(\'' + item.icon_url + '\');"></div>' : "") +
                        '<div class="caption">' + item.caption + '</div>' +
                    '</div>';
        }
        return html;
    }

    this.videoTag = jsp.byClass("video", this.viewDiv);

    this.infoTag = jsp.byClass("info", this.viewDiv);
    this.currentProgramTag = jsp.byClass("play-programs", this.infoTag);
    this.channelTag = jsp.byClass("play-channel", this.infoTag);
    this.clockTag = jsp.byClass("clock", this.infoTag);

    this.channelsTag = jsp.byClass("channels", this.viewDiv);
    this.channelsList = jsp.byClass("list", this.channelsTag);
    this.channelsScrollBtn = jsp.byClass("scroll-btn", this.channelsTag);

    this.programsTag = jsp.byClass("programs", this.viewDiv);
    this.programsTitle = jsp.byClass("title", this.programsTag);
    this.programsList = jsp.byClass("list", this.programsTag);
    this.nextArrow = jsp.byClass("next", this.programsTag);
    this.prevArrow = jsp.byClass("prev", this.programsTag);
    this.programsScrollBtn = jsp.byClass("scroll-btn", this.programsTag);

    this.stb = jsp.byId("duneapi");

    this.runClock();
    this.bindVideoLoader();
    this.bindProgramChangeEvent();
    this.updateChannelPosition();
};

jsp.extend(jsp.views.video, jsp.views.BaseView);

jsp.views.video.prototype.bindVideoLoader = function() {
    var stb = this.stb;

    stb.setColorKey(0xFF0000);

    stb.setPlaybackEventCallback(function(previousState, currentState, lastEvent) {
        switch (currentState) {
            case stb.PLAYBACK_STATE_STOPPED:
                break;
            case stb.PLAYBACK_STATE_INITIALIZING:
                jsp.stat.sendStat(TnsStatuses.READY);
                //document.body.classList.remove("play-video");
                document.body.classList.add("start-load");
                break;
            case stb.PLAYBACK_STATE_PLAYING:
                jsp.stat.sendStat(TnsStatuses.PLAY);
                document.body.classList.add("play-video");
                document.body.classList.remove("start-load");
                break;
            case stb.PLAYBACK_STATE_PAUSED:
                jsp.stat.sendStat(TnsStatuses.PAUSE);
                break;
            case stb.PLAYBACK_STATE_SEEKING:
                break;
            case stb.PLAYBACK_STATE_BUFFERING:
                break;
            case stb.PLAYBACK_STATE_FINISHED:
                break;
            case stb.PLAYBACK_STATE_DEINITIALIZING:
                break;
        }
    }.bind(this));
};

jsp.views.video.prototype.bindProgramChangeEvent = function() {
    var items = this.channelsList.children;
    for (var i = 0, len = items.length; i < len; i++) {
        items[i].addEventListener("blur", this.showProgramInfo.bind(this));
    }
};

jsp.views.video.prototype.updateChannelPosition = function() {
    var MAX_COUNT = 9,
        SCREEN_HEIGHT = 660,
        HEADER_HEIGHT = 52,
        ITEM_HEIGHT = 65;

    if(!this.epgIsVisible) {
        var chCount = jsp.api.getChannelsCount();

        if (chCount > MAX_COUNT) {
            chCount = MAX_COUNT;
        }
        this.channelsTag.style.marginTop = (SCREEN_HEIGHT - HEADER_HEIGHT - ITEM_HEIGHT * chCount)/2 + "px";
    }
};

jsp.views.video.prototype.render = function(channelId) {
    if (!jsp.isVisible(this.videoTag)){
        return;
    }

    this.channel = jsp.api.getChannelInfo(channelId);

    jsp.stat.newStream({stream: {url: this.channel.streaming_url, media: [{url: "dune:" + this.channel.streaming_url, width:1280}] }, width:1280, height:720});

    this.stb.play(this.channel.streaming_url);

    jsp.hide(this.infoTag);
    this.toggleChannelInfo();

    this.markCurrentPlayChannel(channelId);
    this.showCurrentChannelInfo();
    this.hideInfoTimer = setTimeout(this.toggleChannelInfo.bind(this), this.HIDE_INFO_DURATION);
    jsp.api.savePlayChannel({channel: channelId});
    this.videoTag.focus();
};

/* ------------------------------- INFO ABOUT CURRENT CHANNEL --------------------------- */

jsp.views.video.prototype.showCurrentChannelInfo = function() {
    this.updateChannelInfo(this.channel, new Date());
};

jsp.views.video.prototype.showCurrentChannelInfo = function() {
    this.updateChannelInfo(this.channel, new Date());
};

jsp.views.video.prototype.updateChannelInfo = function(channel, dateTime) {
    var currPlay = jsp.api.findProgramByTime(channel.id, dateTime),
        nextPlay = jsp.api.nextProgram(channel.id, currPlay),
        livePlay = jsp.api.findProgramByTime(channel.id, new Date()),
        status = (currPlay == livePlay ? "LIVE" : "Airtime");

    this.channelTag.innerHTML =
        '<div class="numb">' + (channel.number + 1) + '</div>' +
            (channel.icon_url ? '<div class="img" style="background-image: url(\'' + channel.icon_url + '\');"></div>' : "") +
            '<div class="caption">' + channel.caption + '</div>';

    if (this.epgIsVisible) {
        if (currPlay) {

            this.currentProgramTag.innerHTML =
                '<div class="time">' + status + ' ' + currPlay.time + ' - ' + jsp.timeToString(currPlay.stopDate) + '</div>' +
                    '<div class="curr-play">' + (currPlay.caption ? currPlay.caption + '. ' : '') + currPlay.description + '</div>' +
                    '<div class="next-time">NEXT ' + nextPlay.time + ' - ' + jsp.timeToString(nextPlay.stopDate) + '</div>' +
                    '<div class="next-play">' + (nextPlay.caption ? nextPlay.caption + '. ' : '') + nextPlay.description + '</div>';
        } else {
            this.currentProgramTag.innerHTML = this.PROGRAM_NOT_FIND;
        }
    }
};

jsp.views.video.prototype.runClock = function() {
    var self = this;

    //autoupdate the clock
    (function() {
        var dateTime = new Date();
        self.clockTag.innerHTML = jsp.timeToString(dateTime);
        self.clockTimer = setTimeout(arguments.callee, self.CLOCK_UPDATE_DURATION);
    })();
};

jsp.views.video.prototype.toggleChannelInfo = function() {
    clearTimeout(this.hideInfoTimer);
    jsp.isVisible(this.channelsTag) && this.hideChannels();
    jsp.toggle(this.infoTag);
};

/* ------------------------ CHANNELS LIST ------------------------- */

jsp.views.video.prototype.showChannels = function() {
    clearTimeout(this.hideInfoTimer);
    jsp.changeVisible(this.infoTag, this.epgIsVisible);
    jsp.changeVisible(this.programsTag, this.epgIsVisible);
    jsp.show(this.channelsTag);

    this.loadPrograms(this.channel.id);
    this.updateChannelsScroll();
};

jsp.views.video.prototype.hideChannels = function() {
    jsp.hide(this.channelsTag);
    jsp.hide(this.programsTag);
    jsp.hide(this.infoTag);
    this.videoTag.focus();
};

jsp.views.video.prototype.openChannel = function() {
    var channelId = document.activeElement.id;
    if (channelId != this.channel.id) {
        this.render(channelId);
    } else {
        this.videoTag.focus();
        this.hideChannels();
    }
};

jsp.views.video.prototype.changeChannel = function(direction) {
    var channel = jsp.api.getChannelInfo(this.channel.id),
        newChannel = jsp.api.getChannelInfoByNumber(channel.number+direction);

    if (!newChannel) {
        if (direction === 1) {
            newChannel = jsp.api.getFirstChannel();
        } else {
            newChannel = jsp.api.getLastChannel();
        }
    }

    newChannel && this.render(newChannel.id);
};

jsp.views.video.prototype.changeChannelUp = function() {
    this.changeChannel(+1);
};

jsp.views.video.prototype.changeChannelDown = function() {
    this.changeChannel(-1);
};

jsp.views.video.prototype.markCurrentPlayChannel = function(id) {
    var prev = jsp.byClass("play-now", this.channelsTag),
        next = jsp.byId(id);

    prev && prev.classList.remove("play-now");
    next.classList.add("play-now");
};

/* ------------------------ PROGRAMS LIST ------------------------- */

jsp.views.video.prototype.loadPrograms = function(channelId) {
    var el = jsp.byId(channelId),
        prevEl = jsp.byClass("active", this.channelsTag),
        date = new Date();

    prevEl && prevEl.classList.remove("active");
    el.classList.add("active");

    el.focus();
    this.renderPrograms(channelId, date);
    this.updateChannelInfo(jsp.api.getChannelInfo(channelId), date);
};

jsp.views.video.prototype.renderPrograms = function(channelId, date) {
    var items = jsp.api.getProgramsByDate(channelId, date),
        active = jsp.api.findProgramByTime(channelId, new Date()),
        item, html = '';

    if (!items.length && this.programsChannel == channelId) {
        return;
    }

    var nextDayProgram = jsp.api.getProgramsByDate(channelId, jsp.addDaysToDate(date, 1)),
        prevDayProgram = jsp.api.getProgramsByDate(channelId, jsp.addDaysToDate(date, -1));

    jsp.changeVisible(this.nextArrow, !!nextDayProgram.length);
    jsp.changeVisible(this.prevArrow, !!prevDayProgram.length);

    this.programsDate = date;
    this.programsChannel = channelId;

    this.programsTitle.innerHTML = jsp.dateToDayName(date) + ', ' + jsp.dateToString(date);
    this.programsList.scrollTop = 0;

    for (var i = 0, len = items.length; i < len; i++) {
        item = items[i];
        html += '<div tabindex="1" data-enter="hideChannels" data-next="programsNextDay" data-prev="programsPrevDay" data-up="' + (!i ? "programsPrevDay" : "prevProgram") + '" data-down="' + (i == len-1 ? "programsNextDay" : "nextProgram") + '" data-left="focusToChannel" data-time="' + item.startDate.getTime() + '" class="item ' + (item === active? 'active' : '') + '"><div class="time">' + item.time + '</div><div class="description">' + (item.caption ? item.caption + '. ' : '') + item.description + '</div></div>';
    }
    !items.length && (html += this.PROGRAM_NOT_FIND);
    html += '<div class="free-space"></div>';

    this.programsList.innerHTML = html;
    this.updateProgramsScroll();
};

jsp.views.video.prototype.showProgramInfo = function() {
    var element = document.activeElement,
        channel = jsp.api.getChannelInfo(this.programsChannel),
        time = parseInt(element.getAttribute("data-time"), 10);

    time && this.updateChannelInfo(channel, new Date(time));
};

jsp.views.video.prototype.channelsListUp = function() {
    var channel = jsp.api.getChannelInfo(document.activeElement.id),
        newChannel = jsp.api.getChannelInfoByNumber(channel.number-1);

    newChannel && this.loadPrograms(newChannel.id);
    this.updateChannelsScroll();
};

jsp.views.video.prototype.channelsListDown = function() {
    var channel = jsp.api.getChannelInfo(document.activeElement.id),
        newChannel = jsp.api.getChannelInfoByNumber(channel.number+1);

    newChannel && this.loadPrograms(newChannel.id);
    this.updateChannelsScroll();
};

jsp.views.video.prototype.programsNextDay = function() {
    this.renderPrograms(this.programsChannel, jsp.addDaysToDate(this.programsDate, 1));
    this.programsList.children[0].focus();
};

jsp.views.video.prototype.programsPrevDay = function() {
    this.renderPrograms(this.programsChannel, jsp.addDaysToDate(this.programsDate, -1));
    this.programsList.children[0].focus();
};

jsp.views.video.prototype.nextProgram = function() {
    var item = document.activeElement;
    item.nextSibling.focus();
    this.showProgramInfo();
    this.updateProgramsScroll();
};

jsp.views.video.prototype.prevProgram = function() {
    var item = document.activeElement;
    item.previousSibling.focus();
    this.showProgramInfo();
    this.updateProgramsScroll();
};

jsp.views.video.prototype.focusToChannel = function() {
    jsp.byClass("active", this.channelsTag).focus();
};

/* ------------------------- SCROLL PROGRESS --------------------------- */

jsp.views.video.prototype.updateChannelsScroll = function() {
    this.updateScroll(this.channelsList, this.channelsScrollBtn);
};

jsp.views.video.prototype.updateProgramsScroll = function() {
    this.updateScroll(this.programsList, this.programsScrollBtn);
};

jsp.views.video.prototype.updateScroll = function(list, scroll) {
    var SCROLL_MARGIN = 8,
        listHeight = list.offsetHeight,
        scrollHeight = list.scrollHeight;

    if (listHeight < scrollHeight) {
        var btnHeight = (listHeight / scrollHeight) * listHeight,
            scrollPosition = (list.scrollTop / (scrollHeight - listHeight)) * (listHeight - btnHeight);

        scroll.style.height = (btnHeight - SCROLL_MARGIN) + "px";
        scroll.style.top = scrollPosition + "px";
        jsp.show(scroll);
    } else {
        jsp.hide(scroll);
    }
};

/* --------------------------- VOLUME ----------------------------- */

jsp.views.video.prototype.changeVolume = function(value) {
    this.setVolume(this.stb.getVolume() + value);
};

jsp.views.video.prototype.setVolume = function(value) {
    this.stb.setVolume(value);
};

/* -------------------------- EVENT HANDLERS --------------------------- */

jsp.views.video.prototype.empty = function() {
};

jsp.views.video.prototype.keyListener = function(e) {
    var element = document.activeElement;

    switch (e.keyCode) {
        case jsp.keyCode.enter:
            var action = element.getAttribute("data-enter");
            action && this[action] && this[action]();
            break;

        case jsp.keyCode.popUpMenu:
            var action = element.getAttribute("data-popupmenu");
            action && this[action] && this[action]();
            break;

        case jsp.keyCode.info:
            this.showCurrentChannelInfo();
            this.toggleChannelInfo();
            break;

        case jsp.keyCode.down:
            var action = element.getAttribute("data-down");
            if (action && this[action]) {
                this[action]();
                e.preventDefault();
            }
            break;

        case jsp.keyCode.up:
            var action = element.getAttribute("data-up");
            if (action && this[action]) {
                this[action]();
                e.preventDefault();
            }
            break;

        case jsp.keyCode.prev:
            var action = element.getAttribute("data-prev");
            if (action && this[action]) {
                this[action]();
                e.preventDefault();
            }
            break;

        case jsp.keyCode.next:
            var action = element.getAttribute("data-next");
            if (action && this[action]) {
                this[action]();
                e.preventDefault();
            }
            break;

        case jsp.keyCode.left:
            var action = element.getAttribute("data-left");
            if (action && this[action]) {
                this[action]();
                e.preventDefault();
            }
            break;

        case jsp.keyCode.right:
            var action = element.getAttribute("data-right");
            if (action && this[action]) {
                this[action]();
                e.preventDefault();
            }
            break;

        case jsp.keyCode.return:
            if (jsp.isVisible(this.channelsTag)) {
                this.hideChannels();
                e.preventDefault();
            } else if (!jsp.isVisible(this.videoTag)) {
                e.preventDefault();
            }
            break;

        case jsp.keyCode.pPlus:
            this.changeChannelUp();
            break;
        case jsp.keyCode.pMinus:
            this.changeChannelDown();
            break;

        case jsp.keyCode.vPlus:
            this.changeVolume(5);
            break;
        case jsp.keyCode.vMinus:
            this.changeVolume(-5);
            break;
    }
};

jsp.views.video.prototype.destroy = function() {
    clearTimeout(this.clockTimer);
};